package TableUpdaters;


use SOAP::Lite; # + trace;
use strict;
use POSIX qw(setsid);
use POSIX ();
use LWP;
use HTTP::Request;
use HTTP::Response;
use DBI;
use DBD::mysql;
use Carp::Assert;
use IO::File;
use Time::HiRes qw( usleep ualarm gettimeofday tv_interval );


require "Utils.pm";
require "DatabaseInterface.pm";
require "ControllerService.pm";
require "SessionManagementDriver.pm";

my $conffileOK = &Utils::readConfiguration("/etc/TARWIS.conf");
if(!$conffileOK) {
   print "configuration file not found or non-readable, exiting now!\n";
   exit(1);
}

our $ALIVE_STATE_UPDATE_INTERVAL 				= $ENV{"ALIVE_STATE_UPDATE_INTERVAL"};
our $ALIVE_STATE_REQUEST_MAXIMUM_RESPONSE_TIME 	= $ENV{"ALIVE_STATE_REQUEST_MAXIMUM_RESPONSE_TIME"};;

our $MAX_LINES = 1000;


sub AliveStatusUpdater {
	
    # AliveStatusUpdater periodically calls the areNodesAlive function of the SessionManager and then sets 
    # this info in the respective database tables this information is used to display the "alive" state in TARWIS
    
    my $outfile = $_[0];
    my $errfile = $_[1]; 
    
    &Utils::redirectOutput($outfile,$errfile);
    print "starting the AliveStatusUpdater ...  printing output to $outfile\n\n";
    
    $0 = "AliveStatusUpdater";
    
        
    if(!&Utils::isInteger($ENV{"ALIVE_STATE_UPDATE_INTERVAL"})) 	{
		print "invalid setting for ALIVE_STATE_UPDATE_INTERVAL\n";
		exit(1);
	}
	
	elsif($ENV{"ALIVE_STATE_UPDATE_INTERVAL"}<= 0) {
		print "AliveStatusUpdater turned off...exiting\n";
		exit(0);
	}
    
    setpriority(0,0,19);
    
    my $database = new DatabaseInterface();
    my @sensornodes = $database->getAllRegisteredSensorNodes();   
    # only when the service is started, set the value to -1!
    for(my $i=0; $i<@sensornodes; $i++){
        my $node_urn = @sensornodes[$i];
        print "set default alive state of node $node_urn\n";
        $database->setAliveStateOfNode($node_urn,-1);
    }
            
        
    while(1) {
        print "starting alive status update at: " . &Utils::getTimeInMySQLFormat() . "\n";
        
	    my $requestID   = areNodesAliveSOAP(\@sensornodes);
	    print "requestID: $requestID\n";
	   
	    if($requestID != -1) {
	        my $start = time();
		    my $end = time() + $ALIVE_STATE_REQUEST_MAXIMUM_RESPONSE_TIME; 
		    my $done = 0;
		        
		    while($done != 1 && time() < $end ) {
		       for(my $i=0; $i<@sensornodes; $i++){
		          my $node_urn = @sensornodes[$i];
		          my $status = $database->getAsynchronousStateOfNode($requestID, $node_urn); 
		          if( $status == -1 ) {
		             $done = 0;
		             print "node $node_urn state is $status (requestID=$requestID)\n";
		          }
		          else {
		             $done = 1;
		             print "node $node_urn state is $status (requestID=$requestID)\n";
		             $database->setAliveStateOfNode($node_urn, $status);
		          }
		       }
		       if($done){
		           print "done=1\n";
		       }
		       else {
		           my $time_elapsed = (time() - $start);
		           print "(some) nodes have not respondend, wait! time elapsed = $time_elapsed\n";
		           sleep(5);
		       }
		       &Utils::truncateFile($outfile, $MAX_LINES);
		   }
		   print "status check done. exiting while loop\n";		    
	  } #  if req_id
	  else {
	     print "no valid request ID retrieved...\n";
	  }
        
      print "waiting for ALIVE_STATE_UPDATE_INTERVAL = " . $ENV{"ALIVE_STATE_UPDATE_INTERVAL"} . "\n";
      &Utils::truncateFile($outfile, $MAX_LINES);
      &Utils::truncateFile($errfile, $MAX_LINES);
        
      sleep($ALIVE_STATE_UPDATE_INTERVAL);
   } # while 
}




sub areNodesAliveSOAP {
    
   print "areNodesAliveSOAP " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";    
   my @urns = @{$_[0]};
   
   my @urns_soap_request = ();
   
   my $database = new DatabaseInterface();
   
   for(my $i=0; $i<@urns; $i++) {
      # print "areNodesAliveSOAP urn: @urns[$i]\n";
      my $soap_obj = SOAP::Data->name("nodes")->type("")->value(@urns[$i]);
      push(@urns_soap_request, $soap_obj);
   }
   
   
   my $controller_service_url = $ENV{"CONTROLLER_SERVICE_URL"};
   my $controller_url_soaped = SOAP::Data->name("controllerEndpointUrl")->type("")->value($controller_service_url);
     
   my $requestID;
   
   eval{
       print "calling SessionManagementDriver::areNodesAlive " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
       $requestID = &SessionManagementDriver::areNodesAlive(@urns_soap_request, $controller_url_soaped);
       print "requestID obtained: $requestID " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
       
       if( !$requestID eq "") {
          $database->setAsynchronousRequestType($requestID, "areNodesAlive");
          for(my $i=0; $i<@urns; $i++){
              my $node_urn = @urns[$i];
              $database->setAsynchronousStateOfNode($requestID, $node_urn, -1); 
          }
       }
       else {
          $requestID = -1;
       }
   };
   if ($@){
       print "areNodesAliveSOAP: SOAP SessionManagementService seems to be unavailable\n";
       $requestID = -1;
   };
   
    
   print "areNodesAliveSOAP end (req_id=$requestID) " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n"; 
         
   return $requestID;
}



sub CommandExecutor {
    
    
    my $outfile = $_[0];
    my $errfile = $_[1];
    
    &Utils::redirectOutput($outfile, $errfile);
    
    print "starting the CommandExecutor ...  printing output to $outfile\n\n";
    
    $0 = "CommandExecutor";
       
    my $database = new DatabaseInterface();
    
    $database->resetCommandEntries();
    
    my $counter = 0;
    
    while(1){
		my @rows = $database->getNextUndoneCommands();
		my $numrows = scalar @rows;
		if($numrows > 0 && @rows[0]!=-1) {
		    print "numrows = $numrows\n";
		    foreach my $row (@rows) {
		    	my @singlerow = @{$row};
	    		my $urn 	= @singlerow[0];
	    		my $command = @singlerow[1];
	    		my $time 	= @singlerow[2];
	    		my $usecs 	= @singlerow[3];
	    	    my $experiment_id    = @singlerow[4];
	    		my $done 	         = @singlerow[5];
	    		assert($done == 0);
	    		
	    		print "CommandExecutor: entry found time: $time now: " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
				
				if($urn =~ m/broadcast/) {
	   			   my $node = $urn;
	   			   $node =~ s/broadcast\|//;
	   			   if($experiment_id =~ m/NOEXPERIMENT/) {
	                    print "CommandExecutor: no exp_id found!\n";
	   			   }
	   			   else {
	   			   	    my @nodes = $database->getNodesUrnsOfExperiment($experiment_id);  
	   			   	    print "Experiment id: $experiment_id\n";
	   			   	    print "CommandExecutor: request found, BROADCASTING the command $command (scheduled at time: $time) now: " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
	                    print "CommandExecutor: nodes @nodes\n";
	                    &ControllerService::runCommand_execute(\@nodes, $command, $database, $experiment_id);
	   			   }
	   			}
	   			else {
	   			   print "unicast the command: $command!\n";
	   			   print "CommandExecutor: request found, UNICASTING to $urn the command $command (scheduled at time: $time) now: " . &Utils::getTime_Hours_Minutes_Secs_Usecs() . "\n"; 
	   			   my @nodes = ($urn);
	   			   &ControllerService::runCommand_execute(\@nodes, $command, $database, $experiment_id);
	   			}
	   			$database->addUpdateCommandEntry($urn, $command, $time, $usecs, $experiment_id, 1);
		    }
   		}
    	else {
    	    usleep(500000);	
    	    $counter++;
    	    if($counter > 100) {
    	        &Utils::truncateFile($outfile, $MAX_LINES);
                &Utils::truncateFile($errfile, $MAX_LINES);
                $counter = 0;
    	    }
    	}
    	
   	}
}




1;
