<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: I18N.class.inc,v 1.6 2005/02/25 11:16:24 vlry Exp $
 */
/* ****************************************************************** */

/**
*/


if (! defined("I18N_DEFAULT_LOCALE")) {
  /**
  * The default locale to be used if no other locale is set
  */
  define("I18N_DEFAULT_LOCALE",    "en");
}

if (! isset($GLOBALS["I18N_CURRENT_LOCALE"])) {
  /**
  * The current locale 
  */
  $GLOBALS["I18N_CURRENT_LOCALE"] = I18N_DEFAULT_LOCALE;
}


if (! isset($GLOBALS["I18N_MESSAGES"])) {
  /**
  * The global array of localized messages. 
  */
  $GLOBALS["I18N_MESSAGES"] = array();
}


if (! isset($I18N_AVAILABLE_LOCALES)) {
  /**
  * The global array of available locales
  */
  $I18N_AVAILABLE_LOCALES =  array("en" => "English");
}


/**
 * I18N is an utility class which provides static 
 * utility functions for differnent language versions
 * of SNA-Portal.
 *
 *
 * @version $Revision: 1.6 $
 * @package Library 
 *
 * @abstract 
 */ 
class I18N {
  
  /**
  * initializes the current locale according to the following rules
  *
  * <ol>
  *   <li>if $_REQUEST["_locale"] is set, the current locale is set
  *     to the value of this request parameter</li>
  *   <li>if $_SESSION["_locale"] is set, the current locale is set
  *     to the value of this session parameter</li>
  *   <li>if the current locale is set to the default locale
  *     {@link I18N_DEFAULT_LOCALE} </li>
  * </ol>
  *
  *
  * @static
  *
  */
  function initLocale() {
    if (isset($_REQUEST["_locale"])) {
       I18N::setLocale($_REQUEST["_locale"]);
       return;
    }
    if (isset($_SESSION["_locale"])) {
      I18N::setLocale($_SESSION["_locale"]);
      return;
    }
    I18N::setLocale(I18N_DEFAULT_LOCALE);
  }

  /* ****************************************************************** */
  /**
  * Sets the current locale 
  *
  * @param string $locale  the current locale  
  *
  * @static
  */
  function setLocale($locale) {
    $GLOBALS["I18N_CURRENT_LOCALE"] = $locale;
  }

  /* ****************************************************************** */  
  /**
  * Replies the current locale
  *
  * @return string the current locale
  *
  * @static 
  */ 
  function getLocale() {
    $locale= $GLOBALS["I18N_CURRENT_LOCALE"];
    $locale= strtolower($locale);
    return $locale;
  }

  /* ****************************************************************** */
  /**
  * Replies a  localized message. This is function with a variable number
  * of arguments. Since 
  *
  * <ul>
  *  <li>the first argument MUST be present. It is the message key.</li>
  *  <li>there MAY be additional arguments, each of which represents a value
  *    for a placeholder in the localized message</li>
  * </ul>
  * 
  * <b>Examples</b>
  * <pre>
  *    $I18N_MESSAGES["greeting.message"] = "Hello World !";
  *    echo I18N::getMessage("greeting.message");
  *    // ==> output is 'Hello World !'
  * </pre>
  *
  * <pre>
  *    $I18N_MESSAGES["greeting.message"] = "Hello {1}, how are you today ?";
  *    echo I18N::getMessage("greeting.message", "Karl");
  *    // ==> output is 'Hello Karl, how are you today !'
  * </pre>
  *
  * @return string the message in the current locale, all placeholders filled in with
  *   the macro values passed in. 
  *
  * @static 
  */ 
  function getMessage() {
    
    // make sure we have at least one argument: the key of the message
    //
    if (func_num_args() < 1) {
      trigger_error(
        "I18N::expandMessage: at least one argument expected, got '" . func_num_args() . "'",
        E_USER_ERROR 
      );
      return;
    }

    // retrieve the message in the current locale
    //
    $key = func_get_arg(0); 
    if (isset($GLOBALS["I18N_MESSAGES"][$key])) {
       $msg = $GLOBALS["I18N_MESSAGES"][$key];
    } else {
      return false;
    }
    
    // expand makro values
    //
    $tokens = preg_split("/(\{\d+\})/", $msg, -1, PREG_SPLIT_DELIM_CAPTURE);
    $ret= "";
    foreach($tokens as $token) {
      if (preg_match("/^\{(\d+)\}$/", $token, $regs)) {
        $argid = $regs[1];
        if ($argid > func_num_args()) {
           $ret .= "{??}";
        } else {
          $ret .= func_get_arg($argid);
        }
      } else {
        $ret .= $token;
      }
    }
    return $ret;
  }


  /* ****************************************************************** */
  /**
  * Loads a message bundle with a given name for the current locale.
  * 
  * @param string $bundle  the name of the message bundle. This functions loads
  *  the message bundle in the file "$currentLocale/$bundle.messages"
  *
  *
  * @static 
  */
  function loadMessageBundle($bundle) {
    $locale = I18N::getLocale();
    require("i18n/$locale/$bundle.messages");
  }

  /* ****************************************************************** */
  /**
  * Adds an array of key/message pairs to the global array of localized
  * messages.
  *
  * @param array $messages an array of key/message pairs
  *
  * @static 
  */
  function addMessages($messages) {
    if (! is_array($messages) || is_null($messages)) {
      trigger_error(
        "I18N::addMessages: expected array in \$messages, got '$messages'",
        E_USER_ERROR 
      ); 
    }
    $GLOBALS["I18N_MESSAGES"] = array_merge(
       $GLOBALS["I18N_MESSAGES"],
       $messages
    );
  }

  /* ****************************************************************** */
  /**
  * Replies an array of the supported locales. The key of 
  * the array elements is a unique ID for the locale (i.e.
  * 'de' or 'en'). The value is a display name which can
  * be used in UI widgets like drop-down lists.
  *
  * @return array the array of supported locales 
  *
  * @static 
  */ 
  function getSupportedLocales() {
    global $I18N_AVAILABLE_LOCALES;
    return $I18N_AVAILABLE_LOCALES; 
  }

  /* ****************************************************************** */
  /**
  * Replies true, if the locale $locale is supported.  
  *
  * @return bool  true, if the locale $locale is supported; false,
  *   otherwise 
  *
  * @static 
  */ 
  function isSupportedLocale($locale) {
    $locale= strtolower($locale);
    global $I18N_AVAILABLE_LOCALES;
    if (! isset($I18N_AVAILABLE_LOCALES)) {
      return false; 
    }
    return isset($I18N_AVAILABLE_LOCALES[$locale]);
  }

}
