<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 * Copyright (C) 2004-2005  SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: Session.class.inc,v 1.38 2006/09/07 15:19:21 vlry Exp $
 */
/* ****************************************************************** */

/**
 * This maps AAI attribute values supplied in HTTP headers to fields in the
 * PHP session object. 
 *
 * @see ShibbolethHTTPHeaders::storeInSession();
 */
require_once('include/aai/ShibbolethHTTPHeaders.inc');
require_once("include/User.class.inc");
require_once("include/I18N.class.inc");


/**
 * Session is an utility class which provides static 
 * utility functions for dealing with sessions in SNA-Portal .  
 * 
 * @version $Revision: 1.38 $
 * @package Library 
 */ 

class Session {


    /**
     * Creates a new, empty PHP session.
     *
     * @static
     * @global $_SESSION
     */
    function create() {
        Session::start();
        // delete all of the session variables.
        $_SESSION= array();
    }


    /**
     * Starts or resume a PHP session.
     *
     * @static
     * @global $_SESSION
     */
    function start() {
        //XXX: calling session_start() twice throws E_NOTICE
        if(!session_id()){
            session_start();
        }
    }
    
    /**
     * Destroy a PHP session.
     *
     * @static
     * @global $_SESSION
     */
    function destroy() {
        Session::start();
        // delete all of the session variables.
        $_SESSION= array();

        // If it's desired to kill the session, also delete the session cookie.
        if (isset($_COOKIE[session_name()])) {
            setcookie(session_name(), '', time()-42000, '/');
        }

        // If there is a Shibboleth session cookie, destroy it too
        //TODO: shib 1.3 session name is _shibsession_7982794572349759
        $cookies= array_keys($_COOKIE);
        foreach($cookies as $cookie) {
            $pos= strpos($cookie,'_shibsession_');
            if ($pos !== false) {
                setcookie($cookie, '', time()-42000, '/');
            }
        }
        
        // finally destroy session
        session_destroy();
    }
    
    /**
     * Initializes a user session with attributes.
     *
     * Session is started, then Shibboleth attributes or DB attributes are
     * stored in session.
     *
     * @param $user  the user (class User) 
     *
     * @static
     * @global $_SESSION
     */
    function init($user) {

        // start the session
        Session::start();
      
        // set the SESSION['uid']
        Session::setUID($user);
      
        // store WISEBEDaai Shibboleth attribute as trusted user attributes.
        ShibbolethHTTPHeaders::updateAAIAttributes($user);
        
        // set displayName
        Session::setDisplayName($user);
        
        // set locale
        Session::setLocale($user);
        
        // set user privilege
        Session::setPrivilege($user);
    
    }

  /**
   * Checks whether the current User ID is stored in 
   * $_SESSION["userid"]. If not, redirects the user
   * the login page {@link USER_PORTAL_LOGIN_URL}
   *
   *
   * @static
   * @global $_SESSION
   */
  function ensureUserSessionValid() {

    Session::start();
    
    $uid= Session::getUID();
    if (is_null($uid)) {
      header("Location: " . USER_PORTAL_LOGIN_URL);
      exit;   
    }
  }


  /**
   * First starts a session and then checks whether there is a 
   * .
   *
   * If not, redirects the user to the login page and
   * exits. The login page is given by the global constant
   * {@link ADMIN_PORTAL_LOGIN_URL}.
   *
   * @see ADMIN_PORTAL_LOGIN_URL
   *
   * @static
   * @global $_SESSION
   */
  function ensureAdminSessionValid() {
  
    Session::start();
  
    // make sure the current session belongs to an administrator
    // user
    $uid= Session::getUID();
    $admin=& Administrator::reconstituteByUid($uid);
    if (is_null($admin)) {
      header("Location: " . ADMIN_PORTAL_LOGIN_URL);
      exit;
    }
  }


  /**
   * Stores name= value in session.
   * @param $name String variable name
   * @param $value Value
   * 
   * @static
   * @global $_SESSION
   */
  function set($name,$value) {
    $_SESSION[$name]= $value;
  }

  /**
   * Returns the value of variable name stored in session.
   * @param $name String variable name
   * @return $value or NULL if not in session.
   * 
   * @static
   * @global $_SESSION
   */
  function get($name) {
    if (isset($_SESSION[$name])) {
        return $_SESSION[$name];
    }
    else {
        $value= null;
        return $value;
    }  
  }

  /**
   * Checks if a variable is stored in session
   * @param $name String variable name
   * @return true or false.
   * 
   * @static
   * @global $_SESSION
   */
  function exists($name) {
    return isset($_SESSION[$name]);
  }
   
  /**
   * Sets uid (username) of user.
   *
   * @param $user  the user (an instance of class User) 
   *
   * @static
   * @global $_SESSION['uid']
   */
  function setUID($user) {
    $_SESSION['uid']= $user->getUID();
  }
  
  /**
   * Gets uid (username) of user.
   *
   * @return $string The user uid or NULL.
   * @static
   * @global $_SESSION['uid']
   */
  function getUID() {
    if (isset($_SESSION['uid'])) {
        return $_SESSION['uid'];
    }
    else {
        $uid= null;
        return $uid;
    }
  }
  
  
  /**
   * Sets displayName of user.
   *
   * @param $user  the user (an instance of class User) 
   *
   * @static
   * @global $_SESSION['displayName']
   */
  function setDisplayName($user) {
//    $_SESSION['displayName']= $user->getNameOrUid();
    $_SESSION['displayName']= $user->getUserAttribute('commonName');
  }

  /**
   * Sets locale based on user attribute ATID_PREFERREDLANGUAGE.
   *
   * @param $user  the user (an instance of class User) 
   *
   * @static
   * @global $_SESSION['_locale']
   */
  function setLocale($user) {
    $lang= $user->getAttribute(ATID_PREFERREDLANGUAGE);
    if (!is_null($lang)) {
        $lang= trim($lang);
        $lang= strtolower($lang);
        if (I18N::isSupportedLocale($lang)) {
            $_SESSION["_locale"]= $lang;
        }
    }
  }

  /**
   * Sets privilege id and name based on user privilege.
   *
   * @param $user  the user (an instance of class User) 
   *
   * @static
   * @global $_SESSION['privilege']
   * @global $_SESSION['privilegeName']
   */
  function setPrivilege($user) {
    $_SESSION['privilege'] = $user->getPrivilege();
    $_SESSION['privilegeName']  = $user->getUserAttribute('privilegeName');
  }
  
  /* ******************************************************** */
  /**
   * Initializes an admin session with attributes of  
   * an administrator.
   *
   * @param Administrator $admin  the admin user (an instance of class 
   *                       Administrator) 
   *
   * @static
   * @global $_SESSION
   */
  function initAdminSession($admin) {
  
    Session::init($admin);
    
  }

  /* ******************************************************** */
  /**
   * Replies true if the current session is owned by portal
   * administrator. 
   *
   * @return bool true, if the current session is owned by portal
   * administrator. 
   *
   * @static
   * @global $_SESSION
   */
  function isPortalAdministratorSession() {
     return $_SESSION['privilege'] == USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR;
  }

  /* ******************************************************** */
  /**
   * Replies true if the current session is owned by a role
   * administrator.
   *
   * @return bool true, if the current session is owned by
   *   a role administrator.
   *
   * @static
   * @global $_SESSION
   */
  function isRoleAdministratorSession() {
     return $_SESSION['privilege'] == USER_PRIVILEGE_ID_RESOURCE_ADMINISTRATOR;
  }

  /* ******************************************************** */
  /**
   * Replies true if the current session is owned by a role
   * administrator.
   *
   * @return bool true, if the current session is owned by
   *   a role administrator.
   *
   * @static
   * @global $_SESSION
   */
  function isSubscriptionAdministratorSession() {
     return $_SESSION['privilege'] == USER_PRIVILEGE_ID_SUBSCRIPTION_ADMINISTRATOR;
  }

}

?>
