<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: Subscription.class.inc,v 1.35 2007/03/07 14:50:36 vlry Exp $
 */
require_once('include/Event.class.inc');

/** The subscription state ID for a subscription which has not been
 * processed yet by a role owner. 
 */
define("SUBSCRIPTION_STATE_PENDING",   0);
/** 
 * The subscription state ID for a subscription which has been accepted
 * by the respective role owner.
 */
define("SUBSCRIPTION_STATE_ACCEPTED",  1);
/**
 * The subscription state ID for a subscription which has been
 * suspended temporarily by the respective role owner.
 */
define("SUBSCRIPTION_STATE_SUSPENDED", 2);

/**
 * The subscription state ID for a subscription which has been rejected
 * by the respective role owner.
 */
define("SUBSCRIPTION_STATE_REJECTED",  3);


/**
 * This class represents a subscription of a user to a role.
 *
 * 
 * @author Thomas Spreng <spreng@iam.unibe.ch>
 * @version $Revision: 1.35 $
 * @package Library
 */
 
class Subscription {

  /* ***************************************************************** */
  /*                       static functions                            */
  /* ***************************************************************** */

    /**
     * Checks if a subscription exists
     *
     * @param int $user_id The user ID
     * @param int $role_id The role ID
     * @return true if the Subscription exists, false otherwise.
     *
     * @static
     */
    function exists($user_id, $role_id) {
        $exists= false;
        $conn=& MySQLConnection::getConnection();
        $query= 'SELECT subscription_counter FROM Subscription '
              . ' WHERE subscription_user = ' . $user_id
              . '   AND subscription_role = ' . $role_id;           
        $conn->query($query);
        $n_rows= $conn->num_rows();
        $conn->freeresult();
        if ($n_rows == 1) {
            $exists= true;
        }
        return $exists;
    } 

  /**
  * Deletes a subscription from the database
  *
  * @param User  &$user  the owner of the subscription
  * @param Role &$role   the role which the subscription exists for
  * @param int $admin_id the admin id who deleted the subs (default: 1) 
  *
  * @static
  *
  */
  function delete(&$user, &$role, $admin_id= 1) {
    
    if (! $user || ! is_a($user, "User")) {
      trigger_error(
        "expected instanceof User in argument '\$user', got '$user'",
        E_USER_ERROR
      );
      return;
    } 
    if (! $role|| ! is_a($role, "Role")) {
      trigger_error(
        "expected instance of Role in argument '\$role', got '$role'",
        E_USER_ERROR
      );
      return;
    } 
    
    $conn =& MySQLConnection::getConnection();
    // delete the subscription itself
    //
    $query = "DELETE FROM Subscription "
           . " WHERE subscription_user = " . $user->getUserId()
           . "   AND subscription_role = " . $role->getID();           
    $conn->query($query);       

    // event
    $username= $user->getUID();
    $role_title= $role->getTitle();
    $role_id= $role->getID();
    Event::storeSubscriptionEvent("Subscription deleted: user: $username role: $role_title ($role_id)", $admin_id);

  }
  
  
  /* ***************************************************************** */
  /**
   * Replies an array of attribute IDs for
   * required attributes which have not been assigned a value yet
   *
   * @param User  &$user  the owner of the subscription
   * @param Role &$role   the role which the subscription exists for
   *
   * @return array   an array of attribute type IDs. If no attributes are missing,
   *   the array is empty
   *
   * @static 
   */
  function getMissingAttributeIDs($user, $role) {
  
    $conn =& MySQLConnection::getConnection();
  
    $userAttributeTypeIds = $user->getStoredAttributeTypeIds();
    $requiredAttributeTypeIds = $role->getRequiredAttributeIDs();
    if (! $requiredAttributeTypeIds || count($requiredAttributeTypeIds) == 0) {
      return array();
    }
    if (! $userAttributeTypeIds || count($userAttributeTypeIds) == 0) {
      return $requiredAttributeTypeIds;
    } 

    return array_diff($requiredAttributeTypeIds, $userAttributeTypeIds);
  }
  

  /**
   * Reconstitutes a subscription from the database given the user 
   * who owns the subscription and the role the subscription
   * exists for. 
   *
   * @param User  &$user  the owner of the subscription
   * @param Role &$role   the role which the subscription exists for
   *
   * @return Subscription   a reference to the reconstituted subscription, if the subscription
   *    exists; NULL, otherwise 
   *
   * @see MySQLConnection::getConnection()
   *
   * @static 
   *
   */  
  function & reconstitute(&$user, &$role) {

    if (! $user || ! is_a($user, "User")) {
      trigger_error(
        "Subscription::reconstitute: expected instanceof User in argument '\$user', got '$user'",
        E_USER_ERROR
      );
      return;
    } 
    
    if (! $role|| ! is_a($role, "Role")) {
      trigger_error(
        "Subscription::reconstitute: expected instance of Role in argument '\$role', got '$role'",
        E_USER_ERROR
      );
      return;
    } 

    $user_id= $user->getID();
    $role_id= $role->getID();

    $conn=& MySQLConnection::getConnection();
        
    $query  = " SELECT * FROM Subscription ";
    $query .= " WHERE subscription_user = $user_id ";
    $query .= "   AND subscription_role = $role_id";    
    
    $conn->query($query);
    $row = $conn->fetch_array();
    $conn->freeresult();
    
    if ($row) {
      $sub = new Subscription();
      $sub->_setUser(&$user);
      $sub->_setRole(&$role);
      unset($row["subscription_user"]);
      unset($row["subscription_role"]);
      $sub->_setFields(&$row);
    } 
    else {
      $sub= null;
    }
    return $sub; 
    
  }

  /**
   * Reconstitutes a subscription from the database given the user 
   * who owns the subscription and the role the subscription
   * exists for. 
   *
   * @param int  $user_id  the owner of the subscription
   * @param int  $role_id   the role which the subscription exists for
   *
   * @return Subscription   a reference to the reconstituted subscription, if the subscription
   *    exists; NULL, otherwise 
   *
   * @static 
   *
   */  
  function & reconstituteByIDs($user_id, $role_id) {
    $user=& User::reconstituteById($user_id);
    $role=& Role::reconstituteById($role_id);
    
    return Subscription::reconstitute($user,$role);
  }


  /* ***************************************************************** */
  /**
   * Creates a subscription, saves it in the database and replies 
   * an initialized subscription object.
   *
   * @param User  $user   an instance of class User
   * @param Role   $role   an instance of class Role
   * @param int $admin_id the admin id who created the subs (default: 1) 
   *
   * @return Subscription   a reference to the reconstituted subscription, if the subscription
   *    exists; NULL, otherwise 
   *
   * @see MySQLConnection::getConnection()
   *
   * @static 
   *
   */  
  function & create(&$user, &$role, $admin_id= 1) {
    $sub= null;
    if (! $user || ! is_a($user, "User")) {
      trigger_error(
        "Subscription::create: expected instanceof User in argument '\$user', got '$user'",
        E_USER_ERROR
      );
      return $sub;
    } 
    
    if (! $role || ! is_a($role, "Role")) {
      trigger_error(
        "Subscription::create: expected instance of Role in argument '\$role', got '$role'",
        E_USER_ERROR
      );
      return $sub;
    } 
    
    $conn =& MySQLConnection::getConnection();
        
    $query  = " INSERT INTO Subscription ";
    $query .= " (subscription_user, subscription_role, date_created, addr_created) ";
    $query .= " VALUES (";
    $query .= "  " . $user->getUserId() . ", ";
    $query .= "  " . $role->getID() . ", ";
    $query .= "  NOW(), ";
    $remote= '127.0.0.1';
    if (isset($_SERVER['REMOTE_ADDR'])) { $remote= $_SERVER['REMOTE_ADDR']; }
    $query .= "  '$remote' ";
    $query .= ")";
        
    $conn->query($query);

    // event
    $username= $user->getUID();
    $role_title= $role->getTitle();
    $role_id= $role->getID();
    Event::storeSubscriptionEvent("Subscription created: user: $username role: $role_title ($role_id)", $admin_id);
 
    $sub =& Subscription::reconstitute(&$user, &$role);
    
    return $sub;
  }


  /* ***************************************************************** */
  /*                       instance variables                          */
  /* ***************************************************************** */
  
  
  /**
   * the user object 
   *
   * @var User
   * @access private
   */
  var $_user;  
  /**
   * the role object 
   *
   * @var Role
   * @access private
   */
  var $_role;     
  
  /**
   * the database fields 
   *
   * @var Role
   * @access private
   */
  var $_fields;  

  /* ************************************************************* */
  /*               constructor                                     */
  /* ************************************************************* */
  /**
  * constructor
  *
  * @access private
  *
  */
  function Subscription() {
  }

  /* ************************************************************* */
  /*                         methods                               */
  /* ************************************************************* */

  /**
  * Replies a reference to the user who owns this subscription.
  *
  * @return User a reference to the user who owns this  subscription
  */  
  function & getUser() {    
     return $this->_user;
  }

  /**
  * Replies the user_id who owns this subscription.
  *
  * @return int The user_id who owns this  subscription.
  */  
  function getUserID() {    
     return $this->_user->getID();
  }


  /* ************************************************************* */
  /**
  * Sets the user who owns this subscription.
  *
  * @param User $user  the user who owns this subscription
  *
  * @access private
  *
  */  
  function _setUser(&$user) {
    if (! $user || ! is_a($user, "User")) {
      trigger_error(
        "Subscription::_setUser: expected instanceof User in argument '\$user', got '$user'",
        E_USER_ERROR
      );
      return;
    } 
    
    $this->_user =& $user;
  }

  /* ************************************************************* */
  /**
  * Replies a reference to the related role object 
  *
  * @return Role  a reference to the related role 
  */  
  function & getRole() {
     return $this->_role;
  }

  /**
  * Replies the role_id of the related role object 
  *
  * @return int The role_id to the related role 
  */  
  function getRoleID() {
     return $this->_role->getID();
  }

  /* ************************************************************* */
  /**
  * Sets the role this subscription is created for.
  *
  * @param Role $role  the role this subscription is created for.
  *
  * @access private
  *
  */  
  function _setRole(&$role) {
    if (! $role|| ! is_a($role, "Role")) {
      trigger_error(
        "Subscription::reconstitute: expected instance of Role in argument '\$role', got '$role'",
        E_USER_ERROR
      );
      return;
    } 
    
    $this->_role =& $role;
  }
  
  /* ************************************************************* */
  /**
  * Sets the subscription attributes. The attributes consist of an
  * array whose keys are equal to the column names in table 'Subscription'
  *
  * @param array $fields  the subscription attributes  
  *
  * @access private
  *
  */  
  function _setFields(&$fields) {
     if (!is_array($fields) || is_null($fields)) {
       trigger_error(
          "Subscription::_setFields: expected \$fields of type array, not NULL; got '$fields'",
          E_USER_ERROR
       ); 
       return;
     } 
     
     $this->_fields = $fields;
  }

  /* ************************************************************* */
  /**
  * Persists the subscription to the database
  */    
  function persist() {
    
    if (! $this->_user || ! $this->_role) {
       trigger_error(
          "Subscription::persist: illegal state, either user or role not set. Cannot persist this subscription.",
          E_USER_ERROR
       ); 
       return;
    }

    $remote= '';
    if (isset($_SERVER['REMOTE_ADDR'])) { $remote= $_SERVER['REMOTE_ADDR']; }
        
    $conn =& MySQLConnection::getConnection();
        
    $query  = " UPDATE Subscription  ";
    $query .= "   SET subscription_state = " . (int)$this->_fields["subscription_state"] . ", ";
    $query .= "       subscription_role_visited = " . (int)$this->_fields["subscription_role_visited"] . ", ";
    $query .= "       subscription_counter = " . (int)$this->_fields["subscription_counter"] . ", ";
    $query .= "       subscription_last_login = '" . $this->_fields["subscription_last_login"] . "', ";    
    $query .= "       date_modified = NOW(), ";
    $query .= "       addr_modified = '$remote' ";
    $query .= " WHERE subscription_user = " . $this->_user->getID() . " ";
    $query .= "   AND subscription_role = " . $this->_role->getID();    

    $conn->query($query);
    
    if ($conn->affected_rows() != 1) {
       trigger_error(
         "Subscription::persist: failed to update subscription. "
         . "uid=" . $this->_user->getUID()
         . ", rid=" . $this->_role->getID(),
         E_USER_ERROR 
       ); 
    }
  }
  
  /**
   * Sets the Subscription state to ACCEPTED in DB.
   * @param int $admin_id the admin id who accepted the subs (default: 1) 
   */
  function accept($admin_id= 1) {
    $this->setState(SUBSCRIPTION_STATE_ACCEPTED);
    // event
    $username= $this->_user->getUID();
    $role_title= $this->_role->getTitle();
    $role_id= $this->_role->getID();
    Event::storeSubscriptionEvent("Subscription accepted: user: $username role: $role_title ($role_id)", $admin_id);

  }

  /**
   * Sets the Subscription state to REJECTED in DB.
   * @param int $admin_id the admin id who rejected the subs (default: 1) 
   */
  function reject($admin_id= 1) {
    $this->setState(SUBSCRIPTION_STATE_REJECTED);
    // event
    $username= $this->_user->getUID();
    $role_title= $this->_role->getTitle();
    $role_id= $this->_role->getID();
    Event::storeSubscriptionEvent("Subscription rejected: user: $username role: $role_title ($role_id)", $admin_id);

  }

  /**
   * Sets the Subscription state to SUSPENDED in DB.
   * @param int $admin_id the admin id who suspended the subs (default: 1) 
   */
  function suspend($admin_id= 1) {
    $this->setState(SUBSCRIPTION_STATE_SUSPENDED);
    // event
    $username= $this->_user->getUID();
    $role_title= $this->_role->getTitle();
    $role_id= $this->_role->getID();
    Event::storeSubscriptionEvent("Subscription suspended: user: $username role: $role_title ($role_id)",$admin_id);
  }
  

  /**
  * Replies true if the user already accessed the role 
  * represented by this subscription.
  *
  * @return bool true, if the user already accessed the role; false,
  *   otherwise  
  */
  function isVisited() {
    return (boolean)$this->_fields["subscription_role_visited"];
  }

  /* ************************************************************* */
  /**
  * Replies true if the role represented by this subscription is open
  *
  * @return boolean  true, if the role is open ; false,
  *   otherwise  
  */
  function isRoleOpen() {

    $this->_role->reload();
    if ( $this->_role->isOpen() ) return true;
    elseif ($this->_role->isFree() ) return true;
    elseif ($this->_role->isPasswordProtected() ) return true;
    else return false;
  }

  /* ************************************************************* */
  /**
  * Replies true if the role owner did accept the subscription.
  *
  * @return int  1, if the subscription is accepted ; 0,
  *   otherwise  
  */
  function isAccessGranted() {
    return $this->_fields["subscription_state"] == SUBSCRIPTION_STATE_ACCEPTED;
  }

  /* ************************************************************* */
  /**
  * Replies true if the role owner did reject the subscription.
  *
  * @return int  1, if the subscription is rejected ; 0,
  *   otherwise  
  */
  function isAccessBlocked() {
    return $this->_fields["subscription_state"] == SUBSCRIPTION_STATE_REJECTED
        || $this->_fields["subscription_state"] == SUBSCRIPTION_STATE_SUSPENDED;
  }

  /* ************************************************************* */
  /**
  * Marks the subscription as being accepted. 
  *
  * @see setBlocked()
  */
  function setAccessGranted() {
    $this->accept();
  }

  /* ************************************************************* */
  /**
  * Marks the subscription as being rejected. 
  *
  * @see setAccepted()
  */
  function setAccessBlocked() {
    $this->reject();
  }

  /* ************************************************************* */
  /**
  * Increments the login counter for this subscription.
  */
  function updateLogin() {
    $this->_fields["subscription_counter"]++;
    $this->_fields["subscription_last_login"] = strftime("%Y-%m-%d %T");
  }

  /* ************************************************************* */
  /**
  * Replies the value of the login counter for this subscription. 
  *
  * @return int  the login counter  
  */

  function getLoginCounter() {
    return $this->_fields["subscription_counter"];    
  }

  /* ************************************************************* */
  /**
  * Replies the subscription state, i.e. either 
  * {@link SUBSCRIPTION_STATE_PENDING}, {@link SUBSCRIPTION_STATE_ACCEPTED},
  * {@link SUBSCRIPTION_STATE_SUSPENDED}, or {@link SUBSCRIPTION_STATE_REJECTED}
  *
  * @return int  the subscription state 
  *
  */
  function getState() {
    return $this->_fields["subscription_state"];    
  }

  /* ************************************************************* */
  /**
  * Sets the subscription state. $state must be one of the following
  * values:
  * {@link SUBSCRIPTION_STATE_PENDING}, {@link SUBSCRIPTION_STATE_ACCEPTED},
  * {@link SUBSCRIPTION_STATE_SUSPENDED}, or {@link SUBSCRIPTION_STATE_REJECTED}
  *
  * @param int  $state  the subscription state  
  *
  */
  function setState($state) {
    switch($state) {
      case SUBSCRIPTION_STATE_PENDING:
      case SUBSCRIPTION_STATE_ACCEPTED:
      case SUBSCRIPTION_STATE_SUSPENDED:
      case SUBSCRIPTION_STATE_REJECTED:
        // OK, do nothing
        break;
        
      default:
        trigger_error(
           "Subscription::setState: illegal value for \$state, got '$state$",
           E_USER_ERROR
        );
        return;
    }
    $this->_fields["subscription_state"] = $state;  
    
    // update DB
    if (! $this->_user || ! $this->_role) {
       trigger_error(
          "Subscription::setState: Either user or role not set. Cannot update this subscription.",
          E_USER_ERROR
       ); 
       return;
    }
    
    $remote= '127.0.0.1';
    if (isset($_SERVER['REMOTE_ADDR'])) { $remote= $_SERVER['REMOTE_ADDR']; }

    $query= "UPDATE Subscription "
          . "SET subscription_state = $state, "
          . "    date_modified = NOW(), "
          . "    addr_modified = '$remote' "
          . "WHERE subscription_user = " . $this->_user->getID()
          ."   AND subscription_role = " . $this->_role->getID();   

    $conn =& MySQLConnection::getConnection();
    $conn->query($query);    
  }

  /* ************************************************************* */
  /**
  * Replies the date (as MySQL timestamp) the subscription
  * was created. 
  *
  * @return string  the date (as MySQL timestamp) the subscription
  *   was created. 
  */
  function getDateCreated() {
    return $this->_fields["date_created"];    
  }

  /* ************************************************************* */
  /**
  * Replies the date of the last login to this subscribed
  * role. 
  *
  * @return string  the last login date 
  */
  function getLastLogin() {
    return $this->_fields["subscription_last_login"];    
  }

  /* ************************************************************* */
  /**
  * Saves a required attribute for this subscription.
  *
  * @param int $atid  the attribute type ID of the attribute
  * @param string $attributeValue   the attribute value
  * @param bool $isTrusted  true, if the attribute value is trusted; false, 
  *     otherwise 
  * 
  */  
  function persistRequiredAttribute($atid, $attributeValue, $isTrusted=false) {
     
     // make sure $atid is an int
     // 
     if (! is_scalar($atid) || ! $atid > 0) {
       trigger_error(
         "Subscription::saveRequiredAttribute: expected parameter \$atid of type int > 0, got '$atid'",
         E_USER_ERROR
       );
       return; 
     }
     
     // make sure $isTrusted is a boolean
     //
     if (! is_bool($isTrusted)) {
       trigger_error(
         "Subscription::saveRequiredAttribute: expected parameter \$isTrusted of type bool, got '$isTrusted'",
         E_USER_ERROR
       );      
     }
     
     $this->_user->storeAttribute($atid, $attributeValue, $isTrusted);     
  }

  /* ************************************************************* */
  /**
  * Replies the subscription attributes for this subscription
  *
  * @return array  an array of subscription attributes record. Each
  *   record is itself an array. Its keys correspond with the column
  *   names in table Attribute.
  */
  function getSubscriptionAttributes() {
    
     $query  = " SELECT Attribute.* ";
     $query .= " FROM Attribute, Policy ";
     $query .= " WHERE Attribute.attribute_user = " . $this->_user->getUserID() . " ";
     $query .= "   AND Policy.policy_attributetype = Attribute.attribute_type ";
     $query .= "   AND Policy.policy_role = " . $this->_role->getID() . " ";
     $query .= " ORDER BY Attribute.attribute_type";
     
     $conn =& MySQLConnection::getConnection();
     
     $conn->query($query);
     $ret = array();
     $i=0;
     while($row = $conn->fetch_array()) {
       $ret[$i++] = $row; 
     }
     return $ret;     
  }

}


