<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: WebCTIMS_API.class.inc,v 1.14 2007/11/06 09:45:17 vlry Exp $
 */
/* ****************************************************************** */


require_once("include/Logger.class.inc");
require_once("include/webct/WebCTIMS_Constants.inc");
require_once("include/webct/WebCTIMS_Request.class.inc");
require_once("include/webct/WebCTIMS_Response.class.inc");
require_once("include/pre-php-4.2.inc");

/**
* This class is a local proxy to the IMS Enterprise API
* of a remote WebCT instance. The API provides methods
* for creating, updating and exporting objects from the
* WebCT database.  
*
* @version $Revision: 1.14 $
* @access  public
* @package WebCTAPI
*/

class WebCTIMS_API {
   
   /* ***************************************************** */
   /*                  private members                      */
   /* ***************************************************** */
  
   /** 
    * the URL pointing to the remote interface 
    *
    * @var string 
    * @access private
    */ 
   var $_request_url       = "";   

   /** 
    * the shared API secret
    *
    * @var string 
    * @access private
    */ 
   var $_shared_api_secret = false;   
   
   /* ***************************************************** */
   /*                  constructor                          */
   /* ***************************************************** */
   
   /**
    * constructor 
    *
    * Initializes the local proxy with the paramters passed
    * in $params and with default values defined in global
    * constants. 
    *
    * The following keys are allowed in the array $param:
    * <ul>
    *   <li><strong>"WEBCT_IMS_API_REQUEST_URL"</strong> - the
    *       value is the URL of the remote WebCT API </li>
    *   <li><strong>"WEBCT_IMS_SHARED_SECRET"</strong> - the 
    *     shared secret used in the calculation of Message
    *     Authentication Codes</li>
    *   <li><strong>"WEBCT_IMS_SHARED_SECRET_FILE"</strong> - the
    *     name of a file with the 
    *     shared secret used in the calculation of Message
    *     Authentication Codes.</li>
    * </ul>
    *
    * If $params is null or the element "WEBCT_IMS_API_REQUEST_URL"
    * is missing the constructor checks whether there is a constant
    * {@link WEBCT_IMS_API_REQUEST_URL} defined. If not, the request URL
    * is initialized with the value {@link DEFAULT_WEBCT_IMS_API_REQUEST_URL}.
    * <p/>
    *
    * The shared secret is initialized according to the following
    * rules:
    * <ol>
    *   <li>from the parameter "WEBCT_IMS_SHARED_SECRET", if this
    *      parameter is present</li>
    *   <li>from the constant WEBCT_IMS_SHARED_SECRET, if this
    *      constant is defined</li>
    *   <li>from the file given by the parameter "WEBCT_IMS_SHARED_SECRET_FILE", 
    *      if this parameter is present</li>
    *   <li>from the file given by the constant WEBCT_IMS_SHARED_SECRET_FILE, 
    *      if this constant is defined</li>
    *   <li>initialized with the value false</li>
    * </ol>
    * <p/>
    * @param  $params array an arrary with configuration parameters 
    * 
    * @access public
    * @see setRequestUrl()
    * @see setSharedAPISecret()
    *
    */   
   function WebCTIMS_API($params = NULL) {
    
       // initialize the request URL
       //
       if (! is_null($params) && isset($params["WEBCT_IMS_API_REQUEST_URL"])) {
          $this->setRequestUrl($params["WEBCT_IMS_API_REQUEST_URL"]);
       } elseif (defined("WEBCT_IMS_API_REQUEST_URL")) {
          $this->setRequestUrl(WEBCT_IMS_API_REQUEST_URL);
       } else {
          $this->setRequestUrl(NULL);
       }
       
       // initialize the shared secret 
       //
       if (!is_null($params) && isset($params["WEBCT_IMS_SHARED_SECRET"])) {
          $this->setSharedAPISecret($params["WEBCT_IMS_SHARED_SECRET"]);
       } elseif (defined("WEBCT_IMS_SHARED_SECRET")) {
          $this->setSharedAPISecret(WEBCT_IMS_SHARED_SECRET);
       } elseif (!is_null($params)  && isset($params["WEBCT_IMS_SHARED_SECRET_FILE"])) {
          $this->readSharedAPISecret($params["WEBCT_IMS_SHARED_SECRET_FILE"]);
       } elseif (defined("WEBCT_IMS_SHARED_SECRET_FILE")) {
          $this->readSharedAPISecret(WEBCT_IMS_SHARED_SECRET_FILE);
       } else {
          // do nothing - shared API secret is not initialized  
       }             
   }

   

   /* ***************************************************** */
   /*                 setters and getters                   */
   /* ***************************************************** */
   
   /**
   * sets the request URL for the remote interface
   *
   * The API supports the following URL schems:
   * <ul>
   *    <li><code>http</code> and <code>https</code> for accessing 
   *         a remote WebCT server</li>
   *    <li><code>file</code> for debugging and testing</li>
   * </ul>
   *
   * @param string $url  the URL pointing to the remote interface
   *
   */ 
   function setRequestUrl($url) {
      $this->_request_url = $url;
   }
   
   /* ***************************************************** */
   /**
   *  replies the current request URL  
   *  
   * @return string  the current request URL
   *
   */
   function getRequestUrl() {
      return $this->_request_url;
   }
   
   /**
    * @return the hostname component of the request URL
    */
   function getRequestHost() {
	return $this->_getHost();
   }

   /**
    * @return the path component of the request URL
    */
   function getRequestPath() {
	return $this->_getPath();
   }
   
   /* ***************************************************** */
   /**
   *  Reads the shared API secret from a file and saves
   *  it internally.
   *  
   * @param string $filename  the filename
   *
   * @see getSharedAPISecret()
   *
   */
   function readSharedAPISecret($filename) {
     $handle = fopen($filename, "r");
     if (! $handle) {
       trigger_error(
          "WebCTIMS_API::readSharedAPISecret: failed to open shared API secret file '$filename' for reading.",
          E_USER_ERROR
       ); 
     }
     while(!feof($handle)) {
        $secret .= fread($handle, 100);
     }
     fclose($handle); 
     $this->setSharedAPISecret($secret);
   }
   
   /* ***************************************************** */
   /**
   * Replies the shared API secret.
   *  
   * @return string  the shared API secret; false, if the shared
   *  API secret is not defined 
   *
   * @see setSharedAPISecret()
   *
   */   
   function getSharedAPISecret() {
     return $this->_shared_api_secret;
   }

   /* ***************************************************** */
   /**
   * Sets the shared API secret.
   *  
   * @param string  $sharedAPIsecret the shared API secret
   *
   * @see getSharedAPISecret()
   *
   */   
   function setSharedAPISecret($sharedAPIsecret) {
     $this->_shared_api_secret = $sharedAPIsecret;
   }
       
   /* ***************************************************** */
   /**
   * Replies the host part of the request URL. If no request
   * URL is defined replies the host part of the default
   * request URL {@link WEBCT_IMS_DEFAULT_URL}. If the request
   * URL has no host part, replies "localhost".
   *  
   * @return string  the host part of the request URL.
   *
   * @access private
   */      
   function _getHost() {
     $url = $this->getRequestUrl();
     if ( empty( $url ) ) {
       $url = WEBCT_IMS_DEFAULT_URL;
     }  
     $components = parse_url($url);
     if (isset($components["host"])) {
       return $components["host"];  
     } else {
       return "localhost";
     }     
   }

   /**
   * Replies the host part of the request URL. If no request
   * URL is defined replies the port  of the default
   * request URL {@link WEBCT_IMS_DEFAULT_URL}. 
   *
   * If the request URL has no explicit port, determines a default
   * port depending on the URL scheme. Replies 80 for http URLs and
   * 443 for https URLs.
   *
   * Otherwise, replies 0.
   *  
   * @return int the port of the request URL
   *
   * @access private   
   */         
   function _getPort() {
     $url = $this->getRequestUrl();
     
     if (empty($url)) {
       $url =   WEBCT_IMS_DEFAULT_URL;
     }    
     $components = parse_url($url);
     if (isset($components["port"])) {
       return $components["port"];  
     } elseif (isset($components["scheme"]) && strtolower($components["scheme"]) == "http") {
       return 80;
     } elseif (isset($components["scheme"]) && strtolower($components["scheme"]) == "https") {
       return 443;
     } else {
       return 0;
     }          
   }
   
   /* ***************************************************** */
   /**
   * Replies the scheme part of the request URL. If no request
   * URL is defined replies the scheme part of the default
   * request URL {@link WEBCT_IMS_DEFAULT_URL}. 
   *
   * If the request URL has no explicit scheme, replies the
   * default scheme "http".
   *
   * @return string  the scheme of the request URL 
   *
   * @access private   
   */         
   function _getScheme() {
     $url = $this->getRequestUrl();
     if (empty($url)) {
       $url = WEBCT_IMS_DEFAULT_URL;
     }
            
     $components = parse_url($url);
     if (isset($components["scheme"])) {
       return strtolower($components["scheme"]);  
     } else {
       return "http";
     }
   }

   /* ***************************************************** */
   /**
   * Replies the path of the request URL. If no request
   * URL is defined replies the path of the default
   * request URL {@link WEBCT_IMS_DEFAULT_URL}. 
   *
   * If the request URL has no explicit path , replies the
   * default path "/".
   *
   * @return string  the path of the request URL
   *
   * @access private   
   */            
   function _getPath() {
     $url = $this->getRequestUrl();
     if (empty($url)) {
       $url = WEBCT_IMS_DEFAULT_URL;
     }
     $components = parse_url($url);     
     $path = $components["path"]; 
     if (empty($path)) {
         $path = "/"; 
     }
     return $path;
   }

   /* ***************************************************** */
   /*                 private methods                       */
   /* ***************************************************** */

   /**
   * Opens a connection to the IMS API depending on the scheme
   * of the request URL.
   * <ul>
   *   <li>opens a http connection, if scheme is http</li>
   *   <li>opens a SSL connection, if scheme is https</li>
   *   <li>opens a file in write mode, if scheme is file</li>
   * </ul>
   *
   * Triggers an error of type E_USER_WARNING if an error occurs
   * and returns NULL.
   *
   * @return handle  an open file/socket handle; NULL, if an error
   *   occured
   *
   * @access private   
   */            

   function _openConnection() {
      $host = $this->_getHost();
      $port = $this->_getPort();
      $scheme = $this->_getScheme();
      
      $handle = NULL;
      
      if ($scheme == "http") {
         $handle = @fsockopen($host, $port, &$errno, &$error_msg);  
      } elseif ($scheme == "https") {
         $handle = @fsockopen("ssl://$host", $port, &$errno, &$error_msg);  
      } elseif ($scheme == "file") {
         $handle = @fopen($this->_getPath(), "wb");   
      } else {
         trigger_error(
            "WebCTIMSAPI::_open_connection: unsupported URL scheme '" . $scheme . "'",
            E_USER_ERROR
         ); 
         $handle= null;
         return $handle;
      }
      
      if (! $handle) {
        if ($scheme == "http" || $scheme == "https") {
          trigger_error(
            "failed to open connection to URL '" . $this->getRequestUrl() . "'. Details: "
            . "errno=$errno, error_message=$error_msg",
            E_USER_ERROR
          );  
        } elseif ($scheme == "file") {
               trigger_error(
            "failed to open file'" . $this->_getPath() . "' for writing",
            E_USER_ERROR
          );  
        }
        $handle= null;
        return $handle;
      }
      
      return $handle;     
   }

  
   /* ***************************************************** */
   /*                 public methods                        */
   /* ***************************************************** */
   
   function invoke($request) {

      if (! is_object($request) || ! is_a($request, "WebCTIMS_Request")) {
        trigger_error(
           "WebCTIMSAPI::invoke: parameter \$request must be an object. got '$request'",
           E_USER_ERROR
        );
        return false; 
      }
      $secret = $this->getSharedAPISecret();
      if ($secret == false) {
         trigger_error(
           "WebCTIMS_API::invoke: possible configuration error. Shared API secret is not initialized.",
           E_USER_ERROR
         );
      }
      
      $request->setSharedSecret($secret);
      
      $handle = $this->_openConnection();
      if (! $handle) {
        return false; 
      }
      
      // headers
      // set HTTP version according to WEBCT_IMS_REQUEST_HTTP_VERSION constant, default HTTP/1.0
      $header=  "POST " . $this->getRequestPath() . " " . WEBCT_IMS_REQUEST_HTTP_VERSION . "\r\n";
      $header.= "Host: " . $this->getRequestHost() . "\r\n";
      $header.= "UserAgent: SNAportal " . SNA_PORTAL_VERSION;
      if (isset( $_SERVER['SERVER_NAME'] )) {
         $header.= " (" . $_SERVER['SERVER_NAME'] . ")";
      }
      $header.= "\r\n";
     
      //XXX
      $logger=& Logger::getInstance();
      if ($logger->isDebugEnable()) {
          $logger->debug("WebCTIMS_API::invoke:\n" . $header);
      }
      //print "---IMS_API---PRE HEADERS BEGIN---\n";
      //print $header;
      //print "---IMS_API---PRE HEADERS END---\n";
      
      fwrite($handle, $header);            
      $request->writeAsMimeMultipart($handle);
      
      if ($this->_getScheme() == "http" || $this->_getScheme() == "https") {
        $response = new WebCTIMS_Response();
        $response->read($handle);
      }                 
      fclose($handle);
      
      return $response;
   }
  
}

?>
