<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: WebCTIMS_XMLWriter.class.inc,v 1.6 2006/01/24 12:37:42 vlry Exp $
 */
/* ****************************************************************** */

require_once("include/webct/WebCTIMS_Constants.inc");

/**
* WebCTIMS_XMLWriter is a utility class for writing a well-formed and
* properly indenten IMS objects as XML stream. Output is written to an 
* internal string buffer and can be retrieved using the method
* {@link getBuffer() }.
*
* <b>Typical Usage</b>
* <pre>
*   $writer = new WebCTIMS_XMLWriter();
*   
*   $writer->writeIMSRequestHeader();
*
*   $writer->writeOpenTag("enterprise");
*   $writer->incIndentLevel();
*  
*   $writer->writeOpenTag("properties", array("lang" => "en"));
*   $writer->incIndentLevel();
*
*   $writer->writeElementWithText("datasource","My Datasource");
*   $writer->writeElementWithText("datetime","2003-08-15");
*
*   $writer->decIndentLevel();
*   $writer->writeCloseTag("properties");
*
*   $writer->decIndentLevel();
*   $writer->writeCloseTag("enterprise");
*
*   echo $writer->getBuffer();
* </pre>
*
* This creates the XML fragment
* <pre>
*  &lt;?xml version="1.0" encoding="iso-8859-1" &gt;
*  &lt;!DOCTYPE enterprise SYSTEM "IMS-EP01.dtd" &gt;
*  &lt;enterprise&gt;
*    &lt;properties lang="en"&gt;
*      &lt;datasource&gt;My Datasource&lt;/datasource&gt;
*      &lt;datetime&gt;2003-08-15&lt;/datetime&gt;
*    &lt;/properties&gt;
*  &lt;/enterprise&gt;
* </pre>
*
* @version $Revision: 1.6 $
* @package WebCTAPI
*
*/
class WebCTIMS_XMLWriter {

  /* ***************************************************************** */
  /*                    static methods                                 */
  /* ***************************************************************** */
   /**
   * Replaces XML meta characters in $s by  the respective escape sequences.
   *
   * @param string $s  a string with XML meta characters
   *
   * @return string the string with escaped XML meta characters
   *
   * @static 
   */
   
   function escapeXMLMetaChars($s) {
     $ret = "";
     for($i=0; $i < strlen($s); $i++) {
       if ($s[$i] == "<") {
         $ret .= "&lt;";
       } elseif ($s[$i] == ">") {
         $ret .= "&gt;";
       } elseif ($s[$i] == "&") {
         $ret .= "&amp;";
       } elseif ($s[$i] == "\"") {
         $ret .= "&quot;";
       } else {
         $ret .= $s[$i];
       }
     }  
     return $ret;
   }
   /* ***************************************************************** */
   /*                    private fields                                 */
   /* ***************************************************************** */
   
   /**
   * the string buffer
   *
   * @var string
   * @access private
   */ 
   var $_xml = "";
   
   /**
   * the current indentation level
   *
   * @var int
   * @access private
   */
   var $_indentLevel= 0;

   /* ***************************************************************** */
   /*                    constructor                                    */
   /* ***************************************************************** */   
   function WebCTIMS_XMLWriter() {    
   }
   
   
  /* ***************************************************************** */
  /*                    methods                                        */
   /* ***************************************************************** */   
   /**
   * Resets the internal string buffer to the empty string
   *
   */
   function reset() {
     $this->_xml = "";  
   }

   /* ***************************************************************** */   
   /**
   * Replies the current buffer content.
   *
   * @return string  the current buffer content    
   *
   */  
   function getBuffer() {
     return $this->_xml;
   }

   /* ***************************************************************** */   
   /**
   * Increment the current identation level by 1.
   *
   */
   function incIndentLevel() {
     $this->_indentLevel++; 
   }
   
   /* ***************************************************************** */   
   /**
   * Decrements the current identation level by 1.
   *
   */
   function decIndentLevel() {
     $this->_indentLevel--; 
     if ($this->_indentLevel < 0) {
       $this->_indentLevel = 0;
     }
   }
   /* ***************************************************************** */   
   /**
   * Sets the current identation level.
   *
   * @param int $indentLevel  the current identation level
   * 
   */
   function setIndentLevel($indentLevel) {
     $this->_indentLevel = $indentLevel;  
   }

   /* ***************************************************************** */   
   /**
   * Replies the current identation level. 
   *
   * @return intthe current identation level
   * 
   */  
   function getIndentLevel() {
     return $this->_indentLevel;  
   } 

   /* ***************************************************************** */   
   /**
   * Replies an indentation string for the current indentation level
   *
   * @return string an indentation string for the current indentation level
   *
   * @access private 
   * 
   */  
   function _getIndentString() {
     return str_repeat("  ", $this->getIndentLevel());  
   }

   /* ***************************************************************** */   
   /**
   * Appends a string to the end of the buffer.
   *
   * @param string $s  the string 
   *
   * @access private 
   * 
   */    
   function _append($s) {
     $this->_xml .= $s;
   }

   /* ***************************************************************** */   
   /**
   * Appends a string and a newline to the end of the buffer
   *
   * @param string $s  the string 
   *
   * @access private 
   * 
   */    
   function _appendLine($s) {
     $this->_xml .= $s . "\n";
   }

   /* ***************************************************************** */   
   /**
   * Writes an XML opening tag to the buffer.
   *
   * @param string $name  the tag name 
   *
   * @param array  $attributes  an array with the tag attributes 
   *
   */      
   function writeOpenTag($name, $attributes=NULL) {         
     if (! is_null($attributes) && ! is_array($attributes)) {
          trigger_error(
            "expected an array of attributes as second argument, got '$attributes'",
            E_USER_ERROR
          );
          return;
      }             
      $this->_append($this->_getIndentString());
      $this->_append("<");
      $this->_append($name);
      if (! is_null($attributes)) {
        foreach($attributes as $key => $value) {
          $value = WebCTIMS_XMLWriter::escapeXMLMetaChars($value);
          $this->_append(" $key=\"$value\"");
        }
      }
      $this->_append(">");
      $this->_appendLine("");
   }

   /* ***************************************************************** */   
   /**
   * Writes an XML closing tag to the buffer.
   *
   * @param string $name  the tag name 
   *
   */      
   function writeCloseTag($name) {
     $this->_append($this->_getIndentString());
     $this->_appendLine("</$name>");
   }

   /* ***************************************************************** */   
   /**
   * Writes an XML empty tag to the buffer.
   *
   * @param string $name  the tag name 
   *
   * @param array  $attributes  an array with the tag attributes 
   *
   */      
   function writeEmptyTag($name, $attributes=NULL ) {
       if (! is_null($attributes) && ! is_array($attributes)) {
          trigger_error(
            "expected an array of attributes as second argument, got '$attributes'",
            E_USER_ERROR
          );
          return;
      }             
      $this->_append($this->_getIndentString());
      $this->_append("<");
      $this->_append($name);
      if (! is_null($attributes)) {
        foreach($attributes as $key => $value) {
          $value = WebCTIMS_XMLWriter::escapeXMLMetaChars($value);
          $this->_append(" $key=\"$value\"");
        }
      }
      $this->_append("/>");
      $this->_appendLine("");     
   }

   /* ***************************************************************** */   
   /**
   * Writes an XML element consisting of an opening text, a text node
   * as content, and a closing tag to the buffer
   *
   * @param string $name  the tag name 
   *
   * @param string $value  the text content 
   *
   * @param array  $attributes  an array with the tag attributes 
   *
   */      
   function writeElementWithText($name, $textvalue, $attributes=NULL) {
     if (! is_null($attributes) && ! is_array($attributes)) {
          trigger_error(
            "expected an array of attributes as third argument, got '$attributes'",
            E_USER_ERROR
          );
          return;
      }             
      $this->_append($this->_getIndentString());
      $this->_append("<");
      $this->_append($name);
      if (! is_null($attributes)) {
        foreach($attributes as $key => $value) {
          $value = WebCTIMS_XMLWriter::escapeXMLMetaChars($value);
          $this->_append(" $key=\"$value\"");
        }
      }
      $this->_append(">");
      $this->_append(WebCTIMS_XMLWriter::escapeXMLMetaChars($textvalue));
      $this->_append("</$name>");
      $this->_appendLine("");     
   }
  
  
   /**
   * Writes the common header for an IMS object in XML format. 
   * The common header consists of the standard XML prolog 
   * (&lt;?xml version="1.0" encoding="iso-8859-1" &gt;) and
   * of the doctype declaration for the IMS enterprise 
   * object.
   *
   */
   function writeIMSRequestHeader($xml_encoding= null) {
     $this->_appendLine("<?xml version=\"1.0\" encoding=\"iso-8859-1\" ?>");
     $this->_appendLine("<!DOCTYPE enterprise SYSTEM \"IMS-EP01.dtd\" >");
   }
}


?>
