<?php
/*
* SNA-Portal mediates between one or more Authentication and Authorisation
* Infrastructures and non-AAI-enabled roles.
* Copyright (C) 2003 Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
* Copyright (C) 2004 SWITCH (http://www.switch.ch)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
* $Id: WebCTVistaAS_Request.inc,v 1.8 2006/09/07 15:06:30 vlry Exp $
*/
/**
* WebCTAS_Request is the AutoSignon request handler.
*
* @version $Revision: 1.8 $
* @author Valery Tschopp - SWITCH
*
* @package WebCTAPI
*/

require_once("include/webct/WebCTVista.inc");
require_once("include/webct/WebCTVistaAS_Constants.inc");

/**
* The list of WebCT AS paramters taken into consideration for
* the MAC computation.
*
* @var array
* @access private
*/
$_WEBCTVISTA_AS_MAC_PARAM_NAMES = array(
WEBCTVISTA_AS_PARAM_WUUI,
WEBCTVISTA_AS_PARAM_TIMESTAMP,
WEBCTVISTA_AS_PARAM_URL
);

$_WEBCTVISTA_AS_REQUEST_PARAM_NAMES = array(
WEBCTVISTA_AS_PARAM_WUUI,
WEBCTVISTA_AS_PARAM_TIMESTAMP,
WEBCTVISTA_AS_PARAM_URL,
WEBCTVISTA_AS_PARAM_MAC
);	

/**
* WebCTVistaAS_Request represents a request to the WebCT Vista Autosignon API.
*
* <b>Typical Usage</b>
* <pre>
*    // create a request
*    $as_req= new WebCTVistaAS_Request();
*
*    // set URL and secret
*    $as_req->setRequestUrl("http://webctvista.example.org/webct/public/autosignon");
*    $as_req->setSharedSecret("ADSf345aedA86" );
*
*    // set user WUUI and target url
*    $glcid= "";
*    $webctid= "713984793246@example.com";
*    $url= ""http://webctvista.example.org:80/webct/viewMyWebCT.dowebct";
*
*    $as_req->setWUUI( $glcid, $webctid );
*    $as_req->setURL( $url );
*        
*    // redirect browser to autosignon url
*    $location= $as_req->buildAutosignonUrl();
*        
*    header( "Location: $location" );
*    exit;
* </pre>
*
*/
class WebCTVistaAS_Request {
	
	/**
	* the request parameters
	*
	* @var array
	* @access private
	*/
	var $_parameters = array();
	
	
	/**
	* The IMS API shared secret
	*
	* @var string
	* @access private
	*/
	var $_shared_secret = null;
	
	/**
	* The request URL
	*
	* @var string
	* @access private
	*/
	var $_request_url = null;
	
	/**
	* The glcId of institution
	*
	* @var string
	* @access private
	*/
	var $_glcid= null;
	
	/* ***************************************************** */
	/*                  constructor                          */
	/* ***************************************************** */
	
	/**
	* Constructor
	*
	* $apiParameters is an array with  initialization values for the request URL
	* and the shared secret. The request URL is initialized according to the
	* following rules:
	* <ol>
	*   <li>to the value of the parameter "WEBCTVISTA_AS_REQUEST_URL" if this
	*     parameter is set in $apiParameters</li>
	*   <li>to the value of the constant WEBCTVISTA_AS_REQUEST_URL, if this
	*     constant is defined</li>
	*   <li>to false</li>
	* </ol>
	*
	* The shared secret is initialized according to the following
	* rules:
	* <ol>
	*   <li>to the parameter "WEBCTVISTA_AS_SECRET", if this parameter is
	*     set in the array $apiParameters</li>
	*   <li>to the value of the constant WEBCTVISTA_AS_SECRET, if this
	*     constant is defined</li>
	*   <li>to false</li>
	* </ol>
	*
	* @param  array $parameters  an array of request parameters
	*/
	function WebVistaCTAS_Request($parameters = NULL,$apiParameters = NULL) {
		if (! is_null($parameters)) {
			$this->setParameters($parameters);
		}
//		if (is_null($this->getParameter(WEBCTVISTA_AS_PARAM_TIMESTAMP))) {
//			$this->_parameters[WEBCTVISTA_AS_PARAM_TIMESTAMP]= mktime();
//		}
//		
		// init the request URL
		//
		if (! is_null($apiParameters) && isset($apiParameters['WEBCTVISTA_AS_REQUEST_URL'])) {
			$this->setRequestUrl($apiParameters['WEBCTVISTA_AS_REQUEST_URL']);
		} elseif (defined("WEBCTVISTA_AS_REQUEST_URL")) {
			$this->setRequestUrl(WEBCTVISTA_AS_REQUEST_URL);
		}
		
		// init the shared secret
		//
		if (! is_null($apiParameters) && isset($apiParameters["WEBCTVISTA_AS_SECRET"])) {
			$this->setSharedSecret($apiParameters["WEBCTVISTA_AS_SECRET"]);
		} elseif (defined("WEBCTVISTA_AS_SECRET")) {
			$this->setSharedSecret(WEBCTVISTA_AS_SECRET);
		} elseif (! is_null($apiParameters) && isset($apiParameters["WEBCTVISTA_AS_SECRET_FILE"])) {
			$this->readSharedSecret($apiParameters["WEBCTVISTA_AS_SECRET_FILE"]);
		} elseif (defined("WEBCTVISTA_AS_SECRET_FILE")) {
			$this->readSharedSecret(WEBCTVISTA_AS_SECRET_FILE);
		}
		
	}
	
	/**
	* Sets the request URL for the WebCT Autosignon API
	*
	* @param string $url  the URL pointing to the remote interface
	*/
	function setRequestUrl($url) {
		$this->_request_url = $url;
	}
	
	/**
	*  Replies the current request URL
	*
	* @return string  the current request URL
	*/
	function getRequestUrl() {
		return $this->_request_url;
	}
	
	/**
	* sets the value of a  request parameter
	*
	* @param string $name  the name of a request parameter
	* @param  string  $value the value of the request parameter
	*/
	function setParameter($name, $value) {
		$this->_parameters[$name]= $value;
	}
	
	/**
	* Replies the value of a request parameter.
	*
	* @param string $name  the name of a request parameter
	* @return  string the value of the request parameter
	*          or NULL is name doesn't exist
	*/
	function getParameter($name) {
		if (! isset($this->_parameters, $name)) {
            $value= null;
			return $value; // null
		}
		return $this->_parameters[$name];
	}
	
	/**
	* Sets the request parameters.
	*
	* @param  array $params the request parameters
	*/
	function setParameters($params) {
		if (! $params || ! is_array($params)) {
			trigger_error(
			"WebCTVistaAS_Request::setParameters: expected non-null array in parameter \$params, got '$params'",
			E_USER_WARNING
			);
			return;
		}
		$this->_parameters = array();
		foreach($params as $name => $value) {
			$this->_parameters[$name]= $value;
		}
	}
	
	/**
	* Sets the shared secret.
	*
	* @param string $sharedSecret the IMS API shared secret
	*
	*/
	function setSharedSecret($sharedSecret) {
		$this->_shared_secret= $sharedSecret;
	}
	
	/**
	* Replies the shared secret
	*
	* @return string the IMS API shared secret or null if not setted
	*
	*/
	function getSharedSecret() {
		return $this->_shared_secret;
	}
	
	/**
	* Sets the glcId (WebCT Vista global id) of institution
	* to autosignon. The glcid is used to to build the WUUI (WebCT
	* Unique User Id).
	*
	* @param string $glcid 
	*/
	function setGlcId($glcid) {
	   $this->_glcid= $glcid;
	}

	/**
	 * Returns the glcId of user's intitution. Default return null.
	 * 
	 * @return string The glcId of Institution or NULL if glcid not set.
	 */
	function getGlcId() {
	   return $this->_glcid;
	}
	
	/**
	* Sets the target URL 
	*
	* @param string $url 
	*/
	function setURL($url) {
	   $this->_parameters[WEBCTVISTA_AS_PARAM_URL]= $url;
	}
	
	/**
	* Sets the target WebCTID of user. 
	* The WUUI (WebCT Unique User Id) is then build with the institution 
	* glcid and and user webctid in format:
	* <code>$glcid.$webctid</code>.
	*
	* @param string $webctid 
	*/
	function setWebCTID($webctid) {
	    $glcid= $this->getGlcId();
		if ( is_null( $glcid ) ) {
			trigger_error(
			"WebCTVistaAS_Request::setWUUI: glcId is not set. Must be set before this call.",
			E_USER_ERROR
			);
		}
	    $wuui= "$glcid.$webctid";
    	$this->_parameters[WEBCTVISTA_AS_PARAM_WUUI]= $wuui;
	}
	
	/**
	* Sets the WUUI of user. 
	* The WUUI (WebCT Unique User Id) is the ConsortiaId for a particular webctid.
	*
	* @param string $consortia_id The WebCT Vista 3.0 ConsortiaId for webctid.
	*/
	function setWUUI($consortia_id) {
	    $wuui= "$consortia_id";
    	$this->_parameters[WEBCTVISTA_AS_PARAM_WUUI]= $wuui;
	}
	
	/* ***************************************************** */
	/**
	*  Reads the shared API secret from a file and saves
	*  it internally.
	*
	* @param string $filename  the filename
	*
	* @see getSharedSecret()
	*
	*/
	function readSharedSecret($filename) {
		$handle = fopen($filename, "r");
		if (! $handle) {
			trigger_error(
			"WebCTVistaAS_Request::readSharedSecret: failed to open shared API secret file '$filename' for reading.",
			E_USER_ERROR
			);
		}
		while(!feof($handle)) {
			$secret .= fread($handle, 100);
		}
		fclose($handle);
		$this->setSharedSecret($secret);
	}
	
	
	/**
	* Computes the Message
	* Authentication Code (MAC) for this request and sets
	* the respective request parameters.
	*
	* The MAC is saved in the parameter
	* {@link WEBCTVISTTA_AS_PARAM_MAC}. It can be retrieved
	* with the method {@link getParameter() }.
	*
	* @see getParameter()
	*
	* @global $_WEBCTVISTA_AS_MAC_PARAM_NAMES
	*
	*/
	function refreshMAC() {
		global $_WEBCTVISTA_AS_MAC_PARAM_NAMES;
		// set timestamp
		$this->_parameters[WEBCTVISTA_AS_PARAM_TIMESTAMP]= time();
		// build up the MAC data string
		$mac_data= "";
		foreach ($_WEBCTVISTA_AS_MAC_PARAM_NAMES as $param_name) {
			if (isset($this->_parameters[$param_name ])) {
				$mac_data .= $this->_parameters[$param_name];
			}
		}
		
		// compute the MAC
		$secret= $this->_shared_secret;
		if ( is_null( $secret ) ) {
			trigger_error(
			"WebCTVistaAS_Request::refreshMAC: potential configuration error - shared secret is not set",
			E_USER_ERROR
			);
		}
		// compute the MAC
		$mac= WebCTVista::generate_MAC($mac_data, $secret);
		
		// save the MAC as parameter
		$this->_parameters[WEBCTVISTA_AS_PARAM_MAC]= $mac;
	}
	
	
	/* ***************************************************** */
	/**
	* Builds the Autosigon URL the user's browser has to
	* be redirect to in order to log into WebCT.
	*
	* @return string the Autosigon URL
	*
	* @global $_WEBCTVISTA_AS_REQUEST_PARAM_NAMES
	*/
	function buildAutoSignonUrl() {
		// order of names in request
		global $_WEBCTVISTA_AS_REQUEST_PARAM_NAMES;
				
		$this->refreshMAC();		
		$url= $this->getRequestUrl();
		
		$i= 0;
		foreach( $_WEBCTVISTA_AS_REQUEST_PARAM_NAMES as $name ) {
			$url.= ($i++ == 0) ? "?" : "&";
			$value= $this->_parameters[$name];
			$url.= rawurlencode($name) . "=" . rawurlencode($value);
		}
        // add glcId as last param
        $url.= "&glcid=" . $this->getGlcId();
		return $url;
	}
	
}


?>
